@@ -26,7 +26,7 @@
 package global;
 
 import com.sun.tools.javac.api.JavacTaskImpl;
-import com.sun.tools.javac.util.BaseFileObject;
+import com.sun.tools.javac.file.BaseFileObject;
 import java.io.File;
 import java.io.IOException;
 import java.net.URI;
@@ -69,7 +69,7 @@ public class ErrorToleranceTest extends TestCase {
         final String golden = "package test;\n" +
                       "public class Test {\n" +
                       "    private void method(Unknown u) {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : class Unknown\\nlocation: class test.Test\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   class Unknown\\n  location: class test.Test\");" +
                       "    }\n" +
                       "}\n";
 
@@ -108,7 +108,7 @@ public class ErrorToleranceTest extends TestCase {
                       "    private void method(Object o) {\n" +
                       "    }\n" +
                       "    private void method(Unknown u) {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : class Unknown\\nlocation: class test.Test\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   class Unknown\\n  location: class test.Test\");" +
                       "    }\n" +
                       "}\n";
 
@@ -128,10 +128,10 @@ public class ErrorToleranceTest extends TestCase {
         final String golden = "package test;\n" +
                       "public class Test {\n" +
                       "    public Test() {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : variable bflmpsvz\\nlocation: class test.Test\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   variable bflmpsvz\\n  location: class test.Test\");" +
                       "    }\n" +
                       "    public Test(Object o) {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : variable bflmpsvz\\nlocation: class test.Test\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   variable bflmpsvz\\n  location: class test.Test\");" +
                       "    }\n" +
                       "    private String s;\n" +
                       "}\n";
@@ -148,7 +148,7 @@ public class ErrorToleranceTest extends TestCase {
         final String golden = "package test;\n" +
                       "public class Test {\n" +
                       "    static {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : variable bflmpsvz\\nlocation: class test.Test\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   variable bflmpsvz\\n  location: class test.Test\");" +
                       "    }\n" +
                       "    private static String s;\n" +
                       "}\n";
@@ -163,13 +163,14 @@ public class ErrorToleranceTest extends TestCase {
                       "        switch(i) {\n" +
                       "            case Unknown.CONSTANT:\n" +
                       "                break;\n" +
+                      "        }\n" +
                       "    }\n" +
                       "}\n";
 
         final String golden = "package test;\n" +
                       "public class Test {\n" +
                       "    private void method(int i) {\n" +
-                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\nsymbol  : variable CONSTANT\\nlocation: class Unknown\");" +
+                      "        throw new RuntimeException(\"Uncompilable source code - cannot find symbol\\n  symbol:   variable Unknown\\n  location: class test.Test\");" +
                       "    }\n" +
                       "}\n";
 
@@ -269,6 +270,72 @@ public class ErrorToleranceTest extends TestCase {
         compareResults(golden, code);
     }
 
+    public void testDuplicateMethods() throws Exception {
+        final String code = "package test;\n" +
+                      "public class Test {\n" +
+                      "    public void test() {\n" +
+                      "    };\n" +
+                      "    public void test() {\n" +
+                      "    };\n" +
+                      "}\n";
+
+        final String golden = "package test;\n" +
+                      "public class Test {\n" +
+                      "    static {\n" +
+                      "        throw new RuntimeException(\"Uncompilable source code - test() is already defined in test.Test\");\n" +
+                      "    }\n" +
+                      "    public void test() {" +
+                      "    }\n" +
+                      "}\n";
+
+        compareResults(golden, code);
+    }
+
+    public void testDuplicateClasses() throws Exception {
+        final String code = "package test;\n" +
+                      "public class Test {\n" +
+                      "    public static class Nested {\n" +
+                      "    }\n" +
+                      "    public static class Nested {\n" +
+                      "    }\n" +
+                      "}\n";
+
+        final String golden = "package test;\n" +
+                      "public class Test {\n" +
+                      "    static {\n" +
+                      "        throw new RuntimeException(\"Uncompilable source code - test.Test.Nested is already defined in test.Test\");\n" +
+                      "    }\n" +
+                      "    public static class Nested {\n" +
+                      "    }\n" +
+                      "}\n";
+
+        compareResults(golden, code);
+    }
+
+    public void testIssue147516() throws Exception {
+        final String code = "package test;\n" +
+                      "public class Test {\n" +
+                      "    public static final Nested NESTED = new Nested();\n" +
+                      "    public static class Nested implements Runnable {\n" +
+                      "    }\n" +
+                      "}\n";
+
+        final String golden = "package test;\n" +
+                      "public class Test {\n" +
+                      "    static {\n" +
+                      "        throw new RuntimeException(\"Uncompilable source code - test.Test.Nested is not abstract and does not override abstract method run() in java.lang.Runnable\");\n" +
+                      "    }\n" +
+                      "    public static final Nested NESTED = new Nested();\n" +
+                      "    public static class Nested implements Runnable {\n" +
+                      "        static {\n" +
+                      "            throw new RuntimeException(\"Uncompilable source code - test.Test.Nested is not abstract and does not override abstract method run() in java.lang.Runnable\");\n" +
+                      "        }\n" +
+                      "    };\n" +
+                      "}\n";
+
+        compareResults(golden, code);
+    }
+
     //<editor-fold defaultstate="collapsed" desc=" Test Infrastructure ">
     static class MyFileObject extends SimpleJavaFileObject {
         private String text;
@@ -353,7 +420,7 @@ public class ErrorToleranceTest extends TestCase {
         sb.append(clazz.getSuperClass().getClassName());
         sb.append(" implements ");
         for (JavaClass c : clazz.getInterfaces()) {
-            sb.append(c.getSuperClass().getClassName());
+            sb.append(c.getClassName());
             sb.append(", ");
         }
         sb.append("{\n");
