@@ -111,6 +111,11 @@ public class JavacTaskImpl extends JavacTask {
         this(tool, compilerMain, toArray(flags, classes), context, toList(fileObjects));
     }
 
+    JavacTaskImpl(Context context) {
+        this.context = context;
+        setLocale(Locale.getDefault());
+    }
+
     static private String[] toArray(Iterable<String> flags, Iterable<String> classes) {
         ListBuffer<String> result = new ListBuffer<String>();
         if (flags != null)
@@ -173,8 +178,6 @@ public class JavacTaskImpl extends JavacTask {
             if (!filenames.isEmpty())
                 throw new IllegalArgumentException("Malformed arguments " + filenames.toString(" "));
             compiler = JavaCompiler.instance(context);
-            compiler.keepComments = true;
-            compiler.genEndPos = true;
             // NOTE: this value will be updated after annotation processing
             compiler.initProcessAnnotations(processors);
             notYetEntered = new HashMap<JavaFileObject, JCCompilationUnit>();
@@ -273,6 +276,9 @@ public class JavacTaskImpl extends JavacTask {
         assert file != null;
         Enter enter = Enter.instance(context);
         CompilationUnitTree tree = enter.getCompilationUnit(file);
+        if (tree == null && notYetEntered != null) {
+            tree = (CompilationUnitTree) notYetEntered.get(file);
+        }
         return tree;
     }
 
@@ -297,6 +303,10 @@ public class JavacTaskImpl extends JavacTask {
             parsed = true;
             if (compiler != null && compiler.log != null)
                 compiler.log.flush();
+            for (JavaFileObject file : fileObjects) {
+                if (notYetEntered.get(file) == null)
+                    notYetEntered.remove(file);
+            }
         }
     }
 
@@ -777,13 +787,17 @@ public class JavacTaskImpl extends JavacTask {
         }
     }
 
-    public JCBlock reparseMethodBody(CompilationUnitTree topLevel, MethodTree methodToReparse, String newBodyText, int annonIndex) {
+    public JCBlock reparseMethodBody(CompilationUnitTree topLevel, MethodTree methodToReparse, String newBodyText, int annonIndex,
+            final Map<? super JCTree,? super String> docComments) {
         ParserFactory parserFactory = ParserFactory.instance(context);
         CharBuffer buf = CharBuffer.wrap((newBodyText+"\u0000").toCharArray(), 0, newBodyText.length());
-        Parser parser = parserFactory.newParser(buf, ((JCBlock)methodToReparse.getBody()).pos, annonIndex,  ((JCCompilationUnit)topLevel).endPositions);
+        JavacParser parser = (JavacParser) parserFactory.newParser(buf, ((JCBlock)methodToReparse.getBody()).pos, annonIndex,  ((JCCompilationUnit)topLevel).endPositions);
         final JCStatement statement = parser.parseStatement();
         if (statement.getKind() == Tree.Kind.BLOCK) {
-            return (JCBlock) statement;
+            if (docComments != null) {
+                docComments.putAll(parser.getDocComments());
+            }
+            return (JCBlock) statement;            
         }
         return null;
     }
