@@ -400,7 +400,7 @@ public class Flow extends TreeScanner {
     void checkInit(DiagnosticPosition pos, VarSymbol sym) {
         if ((sym.adr >= firstadr || (sym.owner != null && sym.owner.kind != TYP)) &&
             trackable(sym) &&
-            !inits.isMember(sym.adr)) {
+            inits != null && !inits.isMember(sym.adr)) {
             log.error(pos, "var.might.not.have.been.initialized",
                       sym);
             inits.incl(sym.adr);
@@ -793,7 +793,8 @@ public class Flow extends TreeScanner {
         loopPassTwo = prevLoopPassTwo;
         inits = initsWhenFalse;
         uninits = uninitsWhenFalse;
-        alive = alive && !tree.cond.type.isTrue();
+        if (tree.cond.type != null)
+            alive = alive && !tree.cond.type.isTrue();
         alive |= resolveBreaks(tree, prevPendingExits);
     }
 
@@ -810,7 +811,8 @@ public class Flow extends TreeScanner {
             uninitsCond = uninitsWhenFalse;
             inits = initsWhenTrue;
             uninits = uninitsWhenTrue;
-            alive = !tree.cond.type.isFalse();
+            if (tree.cond.type != null)
+                alive = !tree.cond.type.isFalse();
             scanStat(tree.body);
             alive |= resolveContinues(tree);
             if (log.nerrors != 0 ||
@@ -825,7 +827,7 @@ public class Flow extends TreeScanner {
         inits = initsCond;
         uninits = uninitsCond;
         alive = resolveBreaks(tree, prevPendingExits) ||
-            !tree.cond.type.isTrue();
+            tree.cond.type != null && !tree.cond.type.isTrue();
     }
 
     public void visitForLoop(JCForLoop tree) {
@@ -838,7 +840,7 @@ public class Flow extends TreeScanner {
         pendingExits = new ListBuffer<PendingExit>();
         do {
             Bits uninitsEntry = uninits.dup();
-            if (tree.cond != null) {
+            if (tree.cond != null && tree.cond.type != null) {
                 scanCond(tree.cond);
                 initsCond = initsWhenFalse;
                 uninitsCond = uninitsWhenFalse;
@@ -867,7 +869,7 @@ public class Flow extends TreeScanner {
         inits = initsCond;
         uninits = uninitsCond;
         alive = resolveBreaks(tree, prevPendingExits) ||
-            tree.cond != null && !tree.cond.type.isTrue();
+            tree.cond != null && tree.cond.type != null && !tree.cond.type.isTrue();
         nextadr = nextadrPrev;
     }
 
