@@ -131,8 +131,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
      *  unnecessarily deep recursion.
      */
     boolean completionEnabled = true;
-    
-    private JCCompilationUnit disabledCompletionInUnit = null;
 
     /* ---------- Processing import clauses ----------------
      */
@@ -571,36 +569,30 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         // effects of other imports in Resolve.findGlobalType
         Env<AttrContext> localEnv = env.dup(tree);
 
-        try {
-            disabledCompletionInUnit = localEnv.toplevel;
-
-            // Attribute qualifying package or class.
-            JCFieldAccess s = (JCFieldAccess) imp;
-            p = attr.
-                attribTree(s.selected,
-                           localEnv,
-                           tree.staticImport ? TYP : (TYP | PCK),
-                           Type.noType).tsym;
-            if (name == names.asterisk) {
-                // Import on demand.
+        // Attribute qualifying package or class.
+        JCFieldAccess s = (JCFieldAccess) imp;
+        p = attr.
+            attribTree(s.selected,
+                       localEnv,
+                       tree.staticImport ? TYP : (TYP | PCK),
+                       Type.noType).tsym;
+        if (name == names.asterisk) {
+            // Import on demand.
+            chk.checkCanonical(s.selected);
+            if (tree.staticImport)
+                importStaticAll(tree.pos, p, env);
+            else
+                importAll(tree.pos, p, env);
+        } else {
+            // Named type import.
+            if (tree.staticImport) {
+                importNamedStatic(tree.pos(), p, name, localEnv);
                 chk.checkCanonical(s.selected);
-                if (tree.staticImport)
-                    importStaticAll(tree.pos, p, env);
-                else
-                    importAll(tree.pos, p, env);
             } else {
-                // Named type import.
-                if (tree.staticImport) {
-                    importNamedStatic(tree.pos(), p, name, localEnv);
-                    chk.checkCanonical(s.selected);
-                } else {
-                    TypeSymbol c = attribImportType(imp, localEnv).tsym;
-                    chk.checkCanonical(imp);
-                    importNamed(tree.pos(), c, env);
-                }
+                TypeSymbol c = attribImportType(imp, localEnv).tsym;
+                chk.checkCanonical(imp);
+                importNamed(tree.pos(), c, env);
             }
-        } finally {
-            disabledCompletionInUnit = null;
         }
     }
 
@@ -625,6 +617,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                         treeCleaner.scan(tree);
                         tree.sym = (MethodSymbol)e.sym;
                         localEnv = methodEnv(tree, env);
+                        tree.sym.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, tree.sym, tree);
                         tree.sym.flags_field |= FROMCLASS;
                         if (tree.sym.type.tag == FORALL) {
                             for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
@@ -743,8 +736,8 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         }
         if (v == null) {
             v = new VarSymbol(0, tree.name, tree.vartype.type, enclScope.owner);
-            v.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, v, tree);
         }
+        v.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, v, tree);
         tree.sym = v;
         if (tree.init != null) {
             v.flags_field |= HASINIT;
@@ -949,13 +942,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         ClassSymbol c = (ClassSymbol)sym;
         ClassType ct = (ClassType)c.type;
         Env<AttrContext> env = enter.typeEnvs.get(c);
-
-        if (disabledCompletionInUnit == env.toplevel) {
-            // Re-install same completer for next time around and return.
-            sym.completer = this;
-            return;
-        }
-
         JCClassDecl tree = (JCClassDecl)env.tree;
         boolean wasFirst = isFirst;
         isFirst = false;
