@@ -186,6 +186,10 @@ public class Repair extends TreeTranslator {
     @Override
     public void visitVarDef(JCVariableDecl tree) {
         super.visitVarDef(tree);
+        if (!allowEnums && tree.sym != null && (tree.sym.flags_field & Flags.ENUM) != 0) {
+            tree.sym.flags_field &= ~Flags.ENUM;
+            tree.mods.flags &= ~Flags.ENUM;
+        }
         if (hasError) {
             JCTree parent = parents != null ? parents.tail.head : null;
             if (parent != null && parent.getTag() == JCTree.CLASSDEF) {
@@ -218,15 +222,17 @@ public class Repair extends TreeTranslator {
                 tree.body.stats = List.<JCStatement>nil();
         }
         if (hasError && !hadDuplicateSymError) {
-            if (tree.sym != null) {
-                tree.sym.flags_field &= ~(Flags.ABSTRACT | Flags.NATIVE);
-                tree.sym.defaultValue = null;
-            }
-            tree.defaultValue = null;
-            if (tree.body == null) {
+            if (tree.body != null) {
+                if (tree.sym != null)
+                    tree.sym.flags_field &= ~(Flags.ABSTRACT | Flags.NATIVE);
+                tree.body.stats = List.of(generateErrStat(tree.pos(), err != null ? err.getMessage(null) : null));
+            } else if (tree.sym == null || (tree.sym.flags_field & Flags.ABSTRACT) == 0) {
                 tree.body = make.Block(0, List.<JCStatement>nil());
+                tree.body.stats = List.of(generateErrStat(tree.pos(), err != null ? err.getMessage(null) : null));
             }
-            tree.body.stats = List.of(generateErrStat(tree.pos(), err != null ? err.getMessage(null) : null));
+            if (tree.sym != null)
+                tree.sym.defaultValue = null;
+            tree.defaultValue = null;
             hasError = false;
             err = null;
         }
@@ -407,7 +413,16 @@ public class Repair extends TreeTranslator {
                     hasError = true;
                     isErrClass = true;
                 }
+                if ((!allowEnums || isErrClass)
+                        && ((c.flags_field & Flags.ENUM) != 0 || (tree.mods.flags & Flags.ENUM) != 0)) {
+                    c.flags_field &= ~Flags.ENUM;
+                    tree.mods.flags &= ~Flags.ENUM;
+                    hasError = true;
+                    isErrClass = true;
+                    classLevelErrTree = tree;
+                }
                 if (hasError && err != null) {
+                    isErrClass = true;
                     classLevelErrTree = err.getTree();
                     classLevelErrMessage = err.getMessage(null);
                 } else if (c.type.isErroneous() && oldHasError && oldErr != null) {
