@@ -1081,7 +1081,8 @@ public class ClassReader extends ClassFile implements Completer {
     void readClassAttr(ClassSymbol c, Name attrName, int attrLen) {
         if (attrName == names.SourceFile) {
             Name n = readName(nextChar());
-            c.sourcefile = new SourceFileObject(n, c.flatname);
+            if (c.sourcefile == null)
+                c.sourcefile = new SourceFileObject(n, c.flatname);
         } else if (attrName == names.InnerClasses) {
             readInnerClasses(c);
         } else if ((allowGenerics || ideMode) && attrName == names.Signature) {
@@ -2101,13 +2102,16 @@ public class ClassReader extends ClassFile implements Completer {
                 if (c.owner == p)  // it might be an inner class
                     p.members_field.enter(c);
             }
-        } else if (c.classfile != null && ((c.flags_field & seen) == 0 || isSigOverClass(c.classfile, file))) {
+        } else if (c.classfile != null && (c.flags_field & seen) == 0) {
             // if c.classfile == null, we are currently compiling this class
             // and no further action is necessary.
             // if (c.flags_field & seen) != 0, we have already encountered
             // a file of the same kind; again no further action is necessary.
-            if ((c.flags_field & (CLASS_SEEN | SOURCE_SEEN)) != 0)
+            if ((c.flags_field & (CLASS_SEEN | SOURCE_SEEN)) != 0) {
                 c.classfile = preferredFileObject(file, c.classfile);
+            }
+        } else if (c.classfile != null && isSigOverClass(c.classfile, file)) {
+            c.classfile = file;
         }
         c.flags_field |= seen;
     }
@@ -2125,7 +2129,7 @@ public class ClassReader extends ClassFile implements Completer {
     protected JavaFileObject preferredFileObject(JavaFileObject a,
                                            JavaFileObject b) {
 
-        if (preferSource)
+        if (preferSource && !b.getName().toLowerCase().endsWith(".sig"))
             return (a.getKind() == JavaFileObject.Kind.SOURCE) ? a : b;
         else {
             long adate = a.getLastModified();
@@ -2265,6 +2269,20 @@ public class ClassReader extends ClassFile implements Completer {
                     extraFileActions(p, fo);
                 }
             }
+            if (classNamesOraculum != null && location == SOURCE_PATH) {
+                JavaFileObject[] sources = classNamesOraculum.divineSources(p.fullname.toString());
+                if (sources != null) {
+                    for (JavaFileObject fo : sources) {
+                        for (String binaryName : classNamesOraculum.divineClassName(fo)) {
+                            String simpleName = binaryName.substring(binaryName.lastIndexOf(".") + 1);
+                            if (SourceVersion.isIdentifier(simpleName) ||
+                                    simpleName.equals("package-info")) {
+                                includeClassFile(p, fo, binaryName);
+                            }
+                        }
+                    }
+                }
+            }
         }
 
     /** Output for "-verbose" option.
