@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2007 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -52,16 +52,16 @@ public class Code {
     public enum StackMapFormat {
         NONE,
         CLDC {
-            Name getAttributeName(Name.Table names) {
+            Name getAttributeName(Names names) {
                 return names.StackMap;
             }
         },
         JSR202 {
-            Name getAttributeName(Name.Table names) {
+            Name getAttributeName(Names names) {
                 return names.StackMapTable;
             }
         };
-        Name getAttributeName(Name.Table names) {
+        Name getAttributeName(Names names) {
             return names.empty;
         }
     }
@@ -274,8 +274,10 @@ public class Code {
      */
     public static int width(List<Type> types) {
         int w = 0;
-        for (List<Type> l = types; l.nonEmpty(); l = l.tail)
-            w = w + width(l.head);
+        if (types != null) {
+            for (List<Type> l = types; l.nonEmpty(); l = l.tail)
+                w = w + width(l.head);
+        }
         return w;
     }
 
