@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2007 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -61,9 +61,9 @@ public class Gen extends JCTree.Visitor {
     private final Check chk;
     private final Resolve rs;
     private final TreeMaker make;
-    private final Name.Table names;
+    private final Names names;
     private final Target target;
-    private       Type stringBufferTypeInternal;
+    private final Type stringBufferType;
     private final Map<Type,Symbol> stringBufferAppend;
     private Name accessDollar;
     private final Types types;
@@ -92,7 +92,7 @@ public class Gen extends JCTree.Visitor {
     protected Gen(Context context) {
         context.put(genKey, this);
 
-        names = Name.Table.instance(context);
+        names = Names.instance(context);
         log = Log.instance(context);
         syms = Symtab.instance(context);
         chk = Check.instance(context);
@@ -102,6 +102,9 @@ public class Gen extends JCTree.Visitor {
         types = Types.instance(context);
         methodType = new MethodType(null, null, null, syms.methodClass);
         allowGenerics = Source.instance(context).allowGenerics();
+        stringBufferType = target.useStringBuilder()
+            ? syms.stringBuilderType
+            : syms.stringBufferType;
         stringBufferAppend = new HashMap<Type,Symbol>();
         accessDollar = names.
             fromString("access" + target.syntheticNameChar());
@@ -146,20 +149,6 @@ public class Gen extends JCTree.Visitor {
         this.useJsrLocally = false; // reset in visitTry
     }
 
-    private Type stringBufferType() {
-        if (stringBufferTypeInternal == null) {
-            try {
-                syms.stringBuilderType.tsym.complete();
-            } catch (CompletionFailure cf) {
-            }
-            //the target level may be set to 1.5, yet the StringBuild may not be accessible:
-            stringBufferTypeInternal = target.useStringBuilder() && syms.stringBuilderType.tsym.type.tag != ERROR
-                ? syms.stringBuilderType
-                : syms.stringBufferType;
-        }
-        return stringBufferTypeInternal;
-    }
-    
     /** Switches
      */
     private final boolean lineDebugInfo;
@@ -376,7 +365,7 @@ public class Gen extends JCTree.Visitor {
     private boolean isOddAccessName(Name name) {
         return
             name.startsWith(accessDollar) &&
-            (name.byteAt(name.len - 1) & 1) == 1;
+            (name.getByteAt(name.getByteLength() - 1) & 1) == 1;
     }
 
 /* ************************************************************************
@@ -519,17 +508,20 @@ public class Gen extends JCTree.Visitor {
         // If there are class initializers, create a <clinit> method
         // that contains them as its body.
         if (clinitCode.length() != 0) {
-            MethodSymbol clinit = new MethodSymbol(
-                STATIC, names.clinit,
-                new MethodType(
-                    List.<Type>nil(), syms.voidType,
-                    List.<Type>nil(), syms.methodClass),
-                c);
-            c.members().enter(clinit);
+            Symbol clinit = c.members().lookup(names.clinit).sym;
+            if (!(clinit instanceof MethodSymbol)) {
+                clinit = new MethodSymbol(
+                    STATIC, names.clinit,
+                    new MethodType(
+                        List.<Type>nil(), syms.voidType,
+                        List.<Type>nil(), syms.methodClass),
+                    c);
+                c.members().enter(clinit);
+            }
             List<JCStatement> clinitStats = clinitCode.toList();
             JCBlock block = make.at(clinitStats.head.pos()).Block(0, clinitStats);
             block.endpos = TreeInfo.endPos(clinitStats.last());
-            methodDefs.append(make.MethodDef(clinit, block));
+            methodDefs.append(make.MethodDef((MethodSymbol)clinit, block));
         }
         // Return all method definitions.
         return methodDefs.toList();
@@ -1959,10 +1951,10 @@ public class Gen extends JCTree.Visitor {
         /** Make a new string buffer.
          */
         void makeStringBuffer(DiagnosticPosition pos) {
-            code.emitop2(new_, makeRef(pos, stringBufferType()));
+            code.emitop2(new_, makeRef(pos, stringBufferType));
             code.emitop0(dup);
             callMethod(
-                pos, stringBufferType(), names.init, List.<Type>nil(), false);
+                pos, stringBufferType, names.init, List.<Type>nil(), false);
         }
 
         /** Append value (on tos) to string buffer (on tos - 1).
@@ -1980,7 +1972,7 @@ public class Gen extends JCTree.Visitor {
             if (method == null) {
                 method = rs.resolveInternalMethod(tree.pos(),
                                                   attrEnv,
-                                                  stringBufferType(),
+                                                  stringBufferType,
                                                   names.append,
                                                   List.of(t),
                                                   null);
@@ -2011,7 +2003,7 @@ public class Gen extends JCTree.Visitor {
         void bufferToString(DiagnosticPosition pos) {
             callMethod(
                 pos,
-                stringBufferType(),
+                stringBufferType,
                 names.toString,
                 List.<Type>nil(),
                 false);
