@@ -220,7 +220,7 @@ public class Scanner implements Lexer {
     /** Report an error at the given position using the provided arguments.
      */
     private void lexError(int pos, String key, Object... args) {
-        log.error(pos, key, args);
+        log.error(seek + pos, key, args);
         token = ERROR;
         errPos = pos;
     }
@@ -974,6 +974,8 @@ public class Scanner implements Lexer {
     /** Sets the current token.
      */
     public void token(Token token) {
+        this.pos += this.token.name.length() - token.name.length();
+        this.prevEndPos = this.pos;
         this.token = token;
     }
 
