@@ -742,7 +742,7 @@ public class ClassDocImpl extends ProgramElementDocImpl implements ClassDoc {
                 //### I believe it is attempting to compensate for the confused
                 //### convention of including the nested class qualifiers in the
                 //### 'name' of the inner class, rather than the true simple name.
-                innerClasses[i].name().endsWith(className)) {
+                innerClasses[i].name().endsWith("." + className)) {
                 return innerClasses[i];
             } else {
                 ClassDoc innercd = ((ClassDocImpl) innerClasses[i]).searchClass(className);
